import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import { Children, cloneElement, createElement } from 'react';
import { func, element, oneOfType } from 'prop-types';
import lodashOmit from 'lodash/omit';
import useDB from './useDB';
export var propTypes = {
  children: oneOfType([element, func]),
  component: func,
  render: func
};
var propNames = Object.keys(propTypes);
export default (function (useDBOperation, error) {
  return function (_ref) {
    var db = _ref.db,
        children = _ref.children,
        component = _ref.component,
        render = _ref.render,
        otherProps = _objectWithoutProperties(_ref, ["db", "children", "component", "render"]);

    var props = {
      db: useDB(db, error)
    };

    try {
      Object.assign(props, db ? useDBOperation(db, otherProps) : useDBOperation(otherProps));
    } catch (error) {
      if (component || render) {
        throw error;
      }
    }

    if (component) {
      return createElement(component, props);
    }

    if (render) {
      return render(props);
    }

    if (typeof children === 'function') {
      return children.prototype.render ? createElement(children, props) : children(props);
    }

    if (children && Children.count(children)) {
      return cloneElement(children, props);
    }
  };
});
export var omit = function omit(props) {
  return lodashOmit(props, propNames);
};
//# sourceMappingURL=renderProps.js.map