import _objectSpread from "@babel/runtime/helpers/objectSpread";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
import _regeneratorRuntime from "@babel/runtime/regenerator";
import _typeof from "@babel/runtime/helpers/typeof";
import _asyncToGenerator from "@babel/runtime/helpers/asyncToGenerator";
import PouchDB from 'pouchdb';
import find from 'pouchdb-find';
import { collate } from 'pouchdb-collate';
import { matchesSelector } from 'pouchdb-selector-core';
import changesCache from './changesCache';
import useDB from './useDB';
import useListen from './useListen';
PouchDB.plugin(find);
export default function useFind(db) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : db;

  if (arguments.length < 2) {
    db = undefined;
  }

  db = useDB(db, {
    callee: 'useFind',
    example: 'useFind(db, options)'
  });
  var selector = options.selector,
      limit = options.limit,
      skip = options.skip,
      sort = options.sort;
  return useListen(db, options,
  /*#__PURE__*/
  _asyncToGenerator(
  /*#__PURE__*/
  _regeneratorRuntime.mark(function _callee() {
    return _regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            if (!sort) {
              _context.next = 3;
              break;
            }

            _context.next = 3;
            return db.createIndex({
              index: {
                fields: sort.map(function (field) {
                  return _typeof(field) === 'object' ? Object.keys(field)[0] : field;
                })
              }
            });

          case 3:
            _context.next = 5;
            return db.find(options);

          case 5:
            return _context.abrupt("return", _context.sent.docs);

          case 6:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, this);
  })), function (docs, setDocs) {
    var _context2;

    setDocs(docs);

    var mutableDocs = _toConsumableArray(docs); // To find deleted and other non-matching documents, listen all changes and use selector in 'change' event.


    return (_context2 = db, changesCache).call(_context2, {
      live: true,
      include_docs: true,
      since: 'now',
      // Documents are kept in memory. 'complete' event can return an empty array.
      return_docs: false
    },
    /*#__PURE__*/
    function () {
      var _ref3 = _asyncToGenerator(
      /*#__PURE__*/
      _regeneratorRuntime.mark(function _callee2(_ref2) {
        var deleted, doc, index, found, length, _ref5, _ref5$docs, replacementDoc, sortOrders, sortedIndex, _ref7, _ref7$docs, lastDoc, _ref8, _ref8$docs, firstDoc;

        return _regeneratorRuntime.wrap(function _callee2$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                deleted = _ref2.deleted, doc = _ref2.doc;
                index = mutableDocs.findIndex(function (_ref4) {
                  var _id = _ref4._id;
                  return doc._id === _id;
                });
                found = index !== -1; // Document was deleted or it does not match the selector?

                if (!(deleted || selector && !matchesSelector(doc, selector))) {
                  _context3.next = 17;
                  break;
                }

                if (!found) {
                  _context3.next = 15;
                  break;
                }

                // Remove.
                mutableDocs.splice(index, 1);
                length = mutableDocs.length; // At the limit?

                if (!(length + 1 === limit)) {
                  _context3.next = 14;
                  break;
                }

                _context3.next = 10;
                return db.find(_objectSpread({}, options, {
                  limit: 1,
                  skip: (options.skip || 0) + length
                }));

              case 10:
                _ref5 = _context3.sent;
                _ref5$docs = _slicedToArray(_ref5.docs, 1);
                replacementDoc = _ref5$docs[0];

                if (replacementDoc) {
                  docs.push(replacementDoc);
                }

              case 14:
                setDocs(mutableDocs);

              case 15:
                _context3.next = 36;
                break;

              case 17:
                if (found) {
                  // Update.
                  mutableDocs[index] = doc;
                } else {
                  // Create.
                  mutableDocs.push(doc);
                }

                if (sort) {
                  sortOrders = sort.map(function (prop) {
                    return _typeof(prop) === 'object' ? Object.entries(prop)[0] : // Default sort order is 'asc'
                    [prop, 'asc'];
                  });
                  mutableDocs.sort(function (a, b) {
                    var _iteratorNormalCompletion = true;
                    var _didIteratorError = false;
                    var _iteratorError = undefined;

                    try {
                      for (var _iterator = sortOrders[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                        var _step$value = _slicedToArray(_step.value, 2),
                            prop = _step$value[0],
                            order = _step$value[1];

                        var result = collate(a[prop], b[prop]);

                        if (result !== 0) {
                          return order === 'asc' ? result : -result;
                        }
                      }
                    } catch (err) {
                      _didIteratorError = true;
                      _iteratorError = err;
                    } finally {
                      try {
                        if (!_iteratorNormalCompletion && _iterator.return != null) {
                          _iterator.return();
                        }
                      } finally {
                        if (_didIteratorError) {
                          throw _iteratorError;
                        }
                      }
                    }

                    return 0;
                  });
                }

                sortedIndex = mutableDocs.findIndex(function (_ref6) {
                  var _id = _ref6._id;
                  return doc._id === _id;
                }); // Document update, new place supposed to be last, `limit` option is set and limit was reached?

                if (!(found && sortedIndex + 1 === limit)) {
                  _context3.next = 27;
                  break;
                }

                _context3.next = 23;
                return db.find(_objectSpread({}, options, {
                  limit: 1,
                  skip: (options.skip || 0) + sortedIndex
                }));

              case 23:
                _ref7 = _context3.sent;
                _ref7$docs = _slicedToArray(_ref7.docs, 1);
                lastDoc = _ref7$docs[0];

                if ((lastDoc === null || lastDoc === void 0 ? void 0 : lastDoc._id) !== doc._id) {
                  mutableDocs[sortedIndex] = lastDoc;
                }

              case 27:
                if (!(skip && sortedIndex === 0)) {
                  _context3.next = 34;
                  break;
                }

                _context3.next = 30;
                return db.find(_objectSpread({}, options, {
                  limit: 1
                }));

              case 30:
                _ref8 = _context3.sent;
                _ref8$docs = _slicedToArray(_ref8.docs, 1);
                firstDoc = _ref8$docs[0];

                if ((firstDoc === null || firstDoc === void 0 ? void 0 : firstDoc._id) !== doc._id) {
                  mutableDocs[0] = firstDoc;
                }

              case 34:
                if (mutableDocs.length > limit) {
                  mutableDocs.splice(limit);
                }

                setDocs(mutableDocs);

              case 36:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee2, this);
      }));

      return function (_x) {
        return _ref3.apply(this, arguments);
      };
    }());
  });
}
//# sourceMappingURL=useFind.js.map